#!/usr/local/bin/ruby

require 'cgi'
require 'yaml'

require 'cocoron'

require 'page'

$KCODE = 'utf-8'


#
# main routine
#
# Available requests are ...
#   cmd=list;lib=library_name;s=start_index;n=nbook
#   cmd=cover_list;lib=library_name;s=start_index;n=nbook
#   cmd=view;id=id;lib=library
#   cmd=edit;id=id;lib=library_name
#   cmd=stat;lib=library
#   cmd=search
#   cmd=search;libs=lib1;libs=lib2;libs=...;title=...;authors=...;...
#   cmd=remove;id=id;lib=library
#


def create_page(cgi)
   case cgi.get_param('cmd')
   when 'list' then
      lib = load_library(cgi.get_param('lib'))
      start_index = cgi.get_param('s'){ 0 }.to_i
      nbook = cgi.get_param('n'){ 0 }.to_i
      sort = cgi.get_param('sort'){ nil }

      ListPage.new(cgi, lib, start_index, nbook, sort)
   when 'cover_list' then
      lib = load_library(cgi.get_param('lib'))
      start_index = cgi.get_param('s'){ 0 }.to_i
      nbook = cgi.get_param('n'){ 0 }.to_i
      sort = cgi.get_param('sort'){ nil }

      CoverListPage.new(cgi, lib, start_index, nbook, sort)
   when 'view' then
      if cgi.get_param('id').empty? then
         TopPage.new(cgi)
      else
	 lib = load_library(cgi.get_param('lib'), :w)

	 begin
	    id = Cocoron::BookID.parse(cgi.get_param('id').untaint)

            unless lib.has_key?(id.code) then
               book_to_add = id.create_book

	       EditPage.new(cgi, lib, book_to_add)
            else
               ViewPage.new(cgi, lib, lib[id.code])
            end
	 rescue Cocoron::BookID::InvalidBookIDError => err then
            ErrorPage.new(cgi, err.message)
	 end
      end
   when 'rdf' then
      id_str = cgi['id'].first
      RDFPage.new(cgi, id_str)
   when 'rss' then
      mode = cgi['mode'].first
      mode = 'ctime' unless %w(mtime ctime).include?(mode)
      RSSPage.new(cgi, mode)
   when 'search' then
      libs = cgi.get_params('libs').select{|l|
                libpaths.include?(libname2path(l)) }
      crits = SearchPage.compile_query(cgi)

      if libs.empty? or crits.empty? then
         SearchPage.new(cgi)
      else
         SearchResultPage.new(cgi, libs, crits)
      end
   when 'stat'
      lib = load_library(cgi.get_param('lib'))

      StatPage.new(cgi, lib)
   when 'edit' then
      id_str = Cocoron::BookID.parse(cgi.get_param('id').untaint).code
      lib = load_library(cgi.get_param('lib'))

      unless lib.has_key?(id_str) then
	 TopPage.new(cgi)
      else
	 book = lib[id_str]

	 EditPage.new(cgi, lib, book)
      end
   when 'remove' then
      lib = load_library(cgi.get_param('lib'))
      id_str = Cocoron::BookID.parse(cgi.get_param('id').untaint).code

      unless lib.has_key?(id_str) then
         TopPage.new(cgi)
      else
         RemovalConfirmationPage.new(cgi, lib, lib[id_str])
      end
   when 'move' then
      from_lib = load_library(cgi.get_param('from'))
      to_lib = load_library(cgi.get_param('to'))
      id_str = Cocoron::BookID.parse(cgi.get_param('id').untaint).code

      unless from_lib.has_key?(id_str) then
         TopPage.new(cgi)
      else
         MoveConfirmationPage.new(cgi, from_lib[id_str], from_lib, to_lib)
      end
   else
      TopPage.new(cgi)
   end
rescue IndexError,	# for CGI#get_param
       Cocoron::BookID::InvalidBookIDError then
   TopPage.new(cgi)
end



begin
   c = CGI.new
   create_page(c).show
rescue Exception => err then
   ErrorPage.new(c, [err.class, err.message, *err.backtrace].join("\n")).show
end
