#!/usr/local/bin/ruby
require 'cocoron'

ATTRS = %w(title authors publisher edition release_date price
image_url_small image_url_medium image_url_large)

if ARGV.empty? or
      ARGV.first == '-h' or
      ARGV.first == '--help' then
   $stderr.printf <<-"EOS"
#{File.basename($0)} library-path [OPTIONS]

OPTIONS
 #{ATTRS.collect{|a| '-' + a }.join(', ')}:
   DO NOT refetch that information.
   EOS
   exit(ARGV.empty? ? 1 : 0)
end

libpath = ARGV.shift

ARGV.each do |a|
   unless ATTRS.delete(a[1..-1]) then
      $stderr.puts "unknown (or mulptiply-specified) option: `#{a}'"
      exit(1)
   end
end

lib = Cocoron::Library.new(libpath, :w)
$stderr.puts "#{lib.size} books"
i = 1

lib.each do |id, book|
   sleep rand + 1

   $stderr.print i, ' '
   i += 1

   begin
      nbook = book.id.create_book
   rescue Net::HTTPServerException
      sleep 2
      retry
   end
   
   next if nbook.title.empty?

   ATTRS.each do |a|
      book.send("#{a}=", nbook.send(a))
   end

   lib[id] = book
end

puts
